﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 16 $
Author(s): Myrroddin of Llane (Alliance US)
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, AceComm-3.0, AceSerializer-3.0, LibCompress
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 16 $"):match("%d+"))

if not LibStub then error(MAJOR_VERSION.."requires LibStub") end
local lib, oldversion = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local _G = getfenv(0)

LibStub("AceSerializer-3.0"):Embed(lib)
LibStub("AceComm-3.0"):Embed(lib)
local CallbackHandler = LibStub("CallbackHandler-1.0")
local LibCompress = LibStub("LibCompress")

lib.eventFrame = lib.eventFrame or CreateFrame("Frame")
lib.eventFrame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
lib.eventFrame:UnregisterAllEvents()

lib.guildBank = lib.guildBank or {}
local joinedGuild
local playerRankChanged
local logged_in

lib.callbacks = lib.callbacks or CallbackHandler:New(lib)

function lib:OnCommReceived(prefix, msg, distribution, sender)
	if prefix ~= "LGBC" then return end
    if sender == UnitName("player") then return end
	
	local guildName = GetGuildInfo("player")
	local pName = UnitName("player")
	local bankData
	
	local command, value = msg:match("^(%S+) (.*)$")
				
	if command == "REQUEST" then
		local numTabs = lib.guildBank.numTabs
		
		local commTabs = ("TABS %s"):format(tostring(numTabs))
		lib:SendCommMessage("LGBC", commTabs, "WHISPER", sender, "BULK")
		
		for page = 1, numTabs do
			local canView = GetGuildBankTabPermissions(page)
			if canView then
				local serialized_data, serialization_error = lib:Serialize(lib.guildBank[page])
				if serialization_error then
					return print(MAJOR_VERSION .. "Error during serialization:", serialization_error)
				end
					
				local compressed_data, compression_error = LibCompress:Compress(serialized_data)
				if compression_error then
					return print(MAJOR_VERSION .. "Error during compression:", compression_error)
				end
				
				-- Construct a comm string for the compressed data.
				local commString = ("UPDATE_PAGE_%s"):format(page) .. compressed_data
				lib:SendCommMessage("LGBC", commString, "WHISPER", sender, "BULK")
				--[===[@debug@
				print("Sending page %s to %s"):format(page, sender)
				--@end-debug@]===]
			end
		end
		
		local commFunds = ("FUNDS %s"):format(tostring(lib.guildBank.guildFunds))
		lib:SendCommMessage("LGBC", commFunds, "WHISPER", sender, "BULK")		
		
	elseif command == "FUNDS" then
		lib.guildBank.guildFunds = tonumber(value)
		
	elseif command == "TABS" then
		lib.guildBank.numTabs = tonumber(value)
		
	elseif command == "UPDATE_PAGE_1" then
		local canView = GetGuildBankTabPermissions(1)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[1] = deserialized_data
		end
	
	elseif command == "UPDATE_PAGE_2" then
		local canView = GetGuildBankTabPermissions(2)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[2] = deserialized_data
		end
		
	elseif command == "UPDATE_PAGE_3" then
		local canView = GetGuildBankTabPermissions(3)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[3] = deserialized_data
		end
		
	elseif command == "UPDATE_PAGE_4" then
		local canView = GetGuildBankTabPermissions(4)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[4] = deserialized_data
		end
		
	elseif command == "UPDATE_PAGE_5" then
		local canView = GetGuildBankTabPermissions(5)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[5] = deserialized_data
		end
		
	elseif command == "UPDATE_PAGE_6" then
		local canView = GetGuildBankTabPermissions(6)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[6] = deserialized_data
		end
		
	elseif command == "UPDATE_PAGE_7" then
		local canView = GetGuildBankTabPermissions(7)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[7] = deserialized_data
		end
		
	elseif command == "UPDATE_PAGE_8" then
		local canView = GetGuildBankTabPermissions(8)
		if not canView then
			return
		else
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION .. "Error during decompression:",decompression_error)
			end
			
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)
			if deserialization_error then
				return print(MAJOR_VERSION .. "Error during deserialization:", deserialization_error)
			end
			lib.guildBank[8] = deserialized_data
		end
	end
	bankData = lib.guildBank
	lib.callbacks:Fire("GuildBankComm_ContentsChanged", pName, bankData, guildName)
end

lib:RegisterComm("LGBC")

function lib:RequestDataAboutPlayer()
	lib:SendCommMessage("LGBC", "REQUEST", "GUILD", nil, "BULK")
	
	if playerRankChanged then
		-- forget any bank tab info the player should not have
		for page = 1, lib.guildBank.numTabs do
			local canView = GetGuildBankTabPermissions(page)
			if not canView then
				lib.guildBank[page].links = nil
				lib.guildBank[page].stacks = nil
			end
		end
	end
end

function lib.eventFrame:GUILD_ROSTER_UPDATE(unit)
	-- if the player joins a guild
	-- or if the player is in a guild at PLAYER_LOGIN
	-- or is the player's rank changed
	-- we need to request new data
	
	if unit ~= UnitName("player") then return end
	
	if (joinedGuild) or (playerRankChanged) or (logged_in) then
		lib:RequestDataAboutPlayer()
	end
	-- logged_in should be true only for PLAYER_LOGIN() and /reloadUI
	-- GUILD_ROSTER_UPDATE() and GUILD_RANKS_UPDATE() fire at same time when a unit's rank changes
	-- therefore reset playerRankChange in case player's rank changes rapidly and often
	playerRankChanged = false
	logged_in = false
end

function lib.eventFrame:GUILD_RANKS_UPDATE(unit)
	if unit ~= UnitName("player") then return end
	playerRankChanged = true
end

function lib.eventFrame:PLAYER_GUILD_UPDATE()
	if not IsInGuild() then -- player left guild			
		local pName = UnitName("player")
		wipe(lib.guildBank)
		local bankData = lib.guildBank
		lib.callbacks:Fire("GuildBankComm_PlayerGuildless", pName, bankData)
		joinedGuild = false			
		-- guild leaders and unguilded people both have rank 0 just to be confusing
		-- therefore we need to force the issue when leaving a guild
		playerRankChanged = false
	else
		joinedGuild = true
		playerRankChanged = true
	end
end

function lib.eventFrame:GUILDBANKFRAME_OPENED()	
	lib.guildBank.numTabs = GetNumGuildBankTabs()
	lib.guildBank.guildFunds = GetGuildBankMoney()
	--[===[@debug@
	local tab = GetCurrentGuildBankTab()
	print("Guild Bank frame open. Looking at tab "..tostring(tab))
	tab = nil
	--@end-debug@]===]
	
	local name, icon
	for page = 1, lib.guildBank.numTabs do
		name, icon = GetGuildBankTabInfo(page)
		
		if not lib.guildBank[page] then
			lib.guildBank[page] = {
				links = {},  --store the slot item links here
				stacks = {}, --store the slot stack counts here
				name = name,
				texture = icon
			}
		end
	end
end

function lib.eventFrame:GUILDBANK_UPDATE_MONEY()
	lib.guildBank.guildFunds = GetGuildBankMoney()
	--[===[@debug@
	local guildName = GetGuildInfo("player")
	print(guildName.." has "..lib.guildBank.guildFunds.." in copper")
	--@end-debug@]===]
end

function lib.eventFrame:GUILDBANK_UPDATE_TABS()
	lib.guildBank.numTabs = GetNumGuildBankTabs()
end

local num, itemString
function lib.eventFrame:GUILDBANKBAGSLOTS_CHANGED()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				lib.guildBank[page].links[slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				lib.guildBank[page].stacks[slot] = num
			else
				-- that slot is empty
				lib.guildBank[page].links[slot] = nil
				lib.guildBank[page].stacks[slot] = nil
			end
			num, itemString = nil, nil
		end
	end
end

function lib.eventFrame:GUILDBANKFRAME_CLOSED()
	-- we want to filter what the player sees
	-- and only send that rather than the whole of lib.guildBank
	local serialized_data, serialization_error
	local compressed_data, compression_error	
	local numTabs = lib.guildBank.numTabs
	local guildName = GetGuildInfo("player")
	local pName = UnitName("player")
		
	local commTabs = ("TABS %s"):format(tostring(numTabs))
	lib:SendCommMessage("LGBC", commTabs, "GUILD", nil, "BULK")
	--[===[@debug@
	print("Sending number of tabs across comm")
	--@end-debug@]===]
	
	local commFunds = ("FUNDS %s"):format(tostring(lib.guildBank.guildFunds))
	lib:SendCommMessage("LGBC", commFunds, "GUILD", nil, "BULK")
	--[===[@debug@
	print("Sending guild funds across comm")
	--end-debug@
	
	for page = 1, numTabs do
		local canView = GetGuildBankTabPermissions(page)
		if canView then
			local serialized_data, serialization_error = lib:Serialize(lib.guildBank[page])
			if serialization_error then
				return print(MAJOR_VERSION .. "Error during serialization:", serialization_error)
			end
					
			local compressed_data, compression_error = LibCompress:Compress(serialized_data)
			if compression_error then
				return print(MAJOR_VERSION .. "Error during compression:", compression_error)
			end
				
			-- Construct a comm string for the compressed data.
			local commString = ("UPDATE_PAGE_%s"):format(page) .. compressed_data
			lib:SendCommMessage("LGBC", commString, "GUILD", nil, "BULK")
			--@debug@
			print("Sending page %s to Guild"):format(page)
			--@end-debug@]===]
		end
	end
	local bankData = lib.guildBank
	lib.callbacks:Fire("GuildBankComm_ContentsChanged", pName, bankData, guildName)
	--@debug@
	print("Bank frame closed. Firing callback")
	--end-debug@
end

function lib.eventFrame:PLAYER_LOGIN()
	logged_in = true
	if IsInGuild() then
		_G.GuildRoster()
	end
end

function lib:Start()
	lib.eventFrame:RegisterEvent("PLAYER_GUILD_UPDATE")
	lib.eventFrame:RegisterEvent("GUILD_RANKS_UPDATE")
	lib.eventFrame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
	lib.eventFrame:RegisterEvent("GUILDBANKFRAME_OPENED")
	lib.eventFrame:RegisterEvent("GUILDBANKFRAME_CLOSED")
	lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_MONEY")	
	lib.eventFrame:RegisterEvent("PLAYER_LOGIN")
	lib.eventFrame:RegisterEvent("GUILD_ROSTER_UPDATE")
end
lib:Start()